#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_ENDPOINT_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_ENDPOINT_H

#include <dbus-cxx.h>

#include <conexus/endpoint.h>

#ifndef __DBUS_CONEXUS__STARTSTOP_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__STARTSTOP_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::StartStop , int32_t )
#endif


namespace Conexus {
  namespace DBus {

    class EndpointAdapter : public ::DBus::Object
    {
      protected:
        EndpointAdapter( Conexus::Endpoint* adaptee=NULL, const std::string& path="/Conexus/Endpoint"):
          ::DBus::Object(path),
          m_adaptee(adaptee)
        {

          ::DBus::MethodBase::pointer temp_method;

          temp_method = this->create_method<void>( "Conexus.Endpoint", "start", sigc::mem_fun(*this, &EndpointAdapter::start_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.Endpoint", "stop", sigc::mem_fun(*this, &EndpointAdapter::stop_adapter_stub_v) );
          temp_method = this->create_method<bool>( "Conexus.Endpoint", "is_running", sigc::mem_fun(*this, &EndpointAdapter::is_running_adapter_stub_b) );
          temp_method->set_arg_name( 0, "result" );
          temp_method = this->create_method<void,int64_t>( "Conexus.Endpoint", "change_state", sigc::mem_fun(*this, &EndpointAdapter::change_state_adapter_stub_vx) );
          temp_method->set_arg_name( 1, "new_state" );
          m_signal_adapter_data_received_size = this->create_signal<void,uint64_t>("Conexus.Endpoint","data_received_size");
          m_signal_adapter_data_received_size->set_arg_name( 0, "data_size" );
          if ( m_adaptee ) m_signal_adapter_connection_data_received_size = m_adaptee->signal_data_received_size().connect( *m_signal_adapter_data_received_size );
          m_signal_adapter_data_transmitted_size = this->create_signal<void,uint64_t>("Conexus.Endpoint","data_transmitted_size");
          m_signal_adapter_data_transmitted_size->set_arg_name( 0, "data_size" );
          if ( m_adaptee ) m_signal_adapter_connection_data_transmitted_size = m_adaptee->signal_data_transmitted_size().connect( *m_signal_adapter_data_transmitted_size );
          m_signal_adapter_opened = this->create_signal<void>("Conexus.Endpoint","opened");
          if ( m_adaptee ) m_signal_adapter_connection_opened = m_adaptee->signal_opened().connect( *m_signal_adapter_opened );
          m_signal_adapter_closed = this->create_signal<void>("Conexus.Endpoint","closed");
          if ( m_adaptee ) m_signal_adapter_connection_closed = m_adaptee->signal_closed().connect( *m_signal_adapter_closed );
          m_signal_adapter_start_stop = this->create_signal<void,Conexus::StartStop>("Conexus.Endpoint","start_stop");
          m_signal_adapter_start_stop->set_arg_name( 0, "condition" );
          if ( m_adaptee ) m_signal_adapter_connection_start_stop = m_adaptee->signal_start_stop().connect( *m_signal_adapter_start_stop );
        }

      public:

        typedef DBusCxxPointer<EndpointAdapter> pointer;

        static pointer create( Conexus::Endpoint* adaptee=NULL, const std::string& path="/Conexus/Endpoint")
          { return pointer( new EndpointAdapter(adaptee, path)); }

        Conexus::Endpoint* adaptee() { return m_adaptee; }

        void set_adaptee( Conexus::Endpoint* adaptee ) {
          m_adaptee = adaptee;
          m_signal_adapter_connection_data_received_size.disconnect();
          m_signal_adapter_connection_data_transmitted_size.disconnect();
          m_signal_adapter_connection_opened.disconnect();
          m_signal_adapter_connection_closed.disconnect();
          m_signal_adapter_connection_start_stop.disconnect();
          if ( m_adaptee ) m_signal_adapter_connection_data_received_size = m_adaptee->signal_data_received_size().connect( *m_signal_adapter_data_received_size );
          if ( m_adaptee ) m_signal_adapter_connection_data_transmitted_size = m_adaptee->signal_data_transmitted_size().connect( *m_signal_adapter_data_transmitted_size );
          if ( m_adaptee ) m_signal_adapter_connection_opened = m_adaptee->signal_opened().connect( *m_signal_adapter_opened );
          if ( m_adaptee ) m_signal_adapter_connection_closed = m_adaptee->signal_closed().connect( *m_signal_adapter_closed );
          if ( m_adaptee ) m_signal_adapter_connection_start_stop = m_adaptee->signal_start_stop().connect( *m_signal_adapter_start_stop );
        }

        void set_adaptee( Conexus::Endpoint& adaptee ) {
          this->set_adaptee(&adaptee);
        }


      private:

        Conexus::Endpoint* m_adaptee;


        void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee(); }

        void start_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->start(); }
        void stop_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->stop(); }
        bool is_running_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_running(); }
        void change_state_adapter_stub_vx( int64_t new_state ) { this->check_adaptee(); m_adaptee->change_state( new_state); }
        ::DBus::signal<void,uint64_t>::pointer m_signal_adapter_data_received_size;
        sigc::connection m_signal_adapter_connection_data_received_size;
        ::DBus::signal<void,uint64_t>::pointer m_signal_adapter_data_transmitted_size;
        sigc::connection m_signal_adapter_connection_data_transmitted_size;
        ::DBus::signal<void>::pointer m_signal_adapter_opened;
        sigc::connection m_signal_adapter_connection_opened;
        ::DBus::signal<void>::pointer m_signal_adapter_closed;
        sigc::connection m_signal_adapter_connection_closed;
        ::DBus::signal<void,Conexus::StartStop>::pointer m_signal_adapter_start_stop;
        sigc::connection m_signal_adapter_connection_start_stop;


    };

  }
}

#endif
