/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "hostportentry.h"

namespace Conexus
{
  namespace Gtk
  {

    HostPortEntry::HostPortEntry ( Layout layout ):
        m_layout ( layout )
    {
      setup_widget();
    }

    HostPortEntry::HostPortEntry(IPAddressInterface * addr, unsigned lookup, Layout layout):
        m_host( addr, lookup ),
        m_port( addr, lookup )
    {
      setup_widget();
    }

    HostPortEntry::HostPortEntry(IPAddressInterface::pointer addr, unsigned lookup, Layout layout):
        m_host( addr, lookup ),
        m_port( addr, lookup )
    {
      setup_widget();
    }

    HostPortEntry::HostPortEntry(IPv4::Address & addr, unsigned lookup, Layout layout):
        m_host( addr, lookup ),
        m_port( addr, lookup )
    {
      setup_widget();
    }

    HostPortEntry::HostPortEntry(IPv6::Address & addr, unsigned lookup, Layout layout):
        m_host( addr, lookup ),
        m_port( addr, lookup )
    {
      setup_widget();
    }

    HostPortEntry::~HostPortEntry() {}

    HostEntry & HostPortEntry::host_entry( )
    {
      return m_host;
    }

    PortEntry & HostPortEntry::port_entry( )
    {
      return m_port;
    }

    void HostPortEntry::set_address(IPAddressInterface * addr, unsigned lookup)
    {
      m_host.set_address( addr, lookup );
      m_port.set_address( addr, lookup );
    }

    void HostPortEntry::set_address(IPAddressInterface::pointer addr, unsigned lookup)
    {
      m_host.set_address( addr, lookup );
      m_port.set_address( addr, lookup );
    }

    void HostPortEntry::set_address(IPv4::Address & addr, unsigned lookup)
    {
      m_host.set_address( addr, lookup );
      m_port.set_address( addr, lookup );
    }

    void HostPortEntry::set_address(IPv6::Address & addr, unsigned lookup)
    {
      m_host.set_address( addr, lookup );
      m_port.set_address( addr, lookup );
    }

    void HostPortEntry::unset_address()
    {
      m_host.unset_address( );
      m_port.unset_address( );
    }

    HostPortEntry::Layout HostPortEntry::layout( )
    {
      return m_layout;
    }

    void HostPortEntry::set_layout ( Layout layout )
    {
      m_layout = layout;
      while ( this->get_children().size() > 0 )
        this->remove( ** ( this->get_children().begin() ) );

      switch ( m_layout )
      {
        case VERTICAL:
          attach ( m_host_alignment, 0, 1, 0, 1, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          attach ( m_port_alignment, 0, 1, 1, 2, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          break;
        case HORIZONTAL:
          attach ( m_host_alignment, 0, 1, 0, 1, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          attach ( m_port_alignment, 1, 2, 0, 1, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          break;
      }
    }

    void HostPortEntry::setup_widget( )
    {
      m_host_alignment.set ( ::Gtk::ALIGN_LEFT, ::Gtk::ALIGN_TOP );
      m_port_alignment.set ( ::Gtk::ALIGN_LEFT, ::Gtk::ALIGN_TOP );
      m_host_alignment.add ( m_host );
      m_port_alignment.add ( m_port );
      this->set_layout(m_layout);
    }

  }

}

