/***************************************************************************
 *   Copyright (C) 2009 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "netcombobox.h"

#include <iostream>
#include <sstream>

#include <conexus/network_device.h>

namespace Conexus
{
  namespace Gtk
  {

    NetComboBox::NetComboBox()
    {
      m_reftreemodel = ::Gtk::ListStore::create ( m_modelcolumns );
      set_model ( m_reftreemodel );
      pack_start ( m_modelcolumns.device );
      pack_start ( m_modelcolumns.description );

      load_from_enumerated();

    }


    NetComboBox::~NetComboBox()
    {
    }

    void NetComboBox::load_from_enumerated( )
    {
      ::Gtk::TreeModel::Row row;

      m_reftreemodel->clear();

      row = * ( m_reftreemodel->append() );
      row[m_modelcolumns.description] = "No device selected";

      std::vector<std::pair<std::string,std::string> > devices;

      devices = Network::interface_ip_addresses();

      for ( size_t i=0; i < devices.size(); i++ )
      {
        row = * ( m_reftreemodel->append() );
        row[m_modelcolumns.description] = devices[i].first;
        row[m_modelcolumns.device] = devices[i].second;
      }

      if ( get_active_row_number() == -1 ) set_active ( 0 );
    }

    void NetComboBox::set_device ( const std::string& device )
    {
      ::Gtk::TreeModel::Row row;
      bool set = false;

      ::Gtk::TreeModel::iterator iter;
      for ( iter = m_reftreemodel->children().begin(); !set && iter != m_reftreemodel->children().end(); iter++ )
      {
        if ( ( *iter ) [m_modelcolumns.device] == device )
        {
          set_active ( iter );
          set = true;
        }
      }
      if ( !set && device.size() > 0 )
      {
        row = * ( m_reftreemodel->append() );
        row[m_modelcolumns.device] = device;
        set_active ( m_reftreemodel->children().end()-1 );
      }
    }

    std::string NetComboBox::device( )
    {
      Glib::ustring device;
      if ( get_active_row_number() <= 0 )
        return std::string();
      device = get_active()->operator[] ( m_modelcolumns.device );
      return device;
    }

  }

}
