/***************************************************************************
 *   Copyright (C) 2001,2002,2003,2004,2005,2006,2007,2008,2009            *
 *   by Rick L. Vinyard, Jr.                                               *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSNETWORK_DEVICE_H
#define CONEXUSNETWORK_DEVICE_H

#include <string>
#include <vector>

#include <sys/ioctl.h>
#include <net/if.h>

namespace Conexus
{
  namespace Network
  {

    unsigned interface_index( const std::string& name );
    std::string interface_name( unsigned ifindex );
    std::vector<std::string> interface_names();
    std::vector<std::pair<unsigned,std::string> > interfaces();
    std::vector<std::pair<std::string, std::string> > interface_ip_addresses();

    /**
     * @todo TODO add set functions for flags
     * @author Rick L Vinyard Jr
     */
    class Device
    {
      public:

        Device( const std::string& name );

        Device( int index );

        virtual ~Device();

        int index();

        void set_index( int index );

        std::string name();

        void set_name( const std::string& name );

        bool is_up();

        bool has_broadcast();

        bool is_debug();

        bool is_loopback();

        bool is_point_to_point();

        bool is_running();

        bool no_arp();

        bool is_promiscuous();

        bool no_trailers();

        bool receives_all_multicast();

        bool is_master();

        bool is_slave();

        bool is_multicast_capable();

        bool port_selectable();

        bool auto_media_active();

        bool is_dynamic();

        int mtu();

        void set_mtu(int mtu);

      protected:

        std::string m_name;

        int m_index;

        struct ifreq m_ifreq;

    };
    
  }
}

#endif
